@@ -49,6 +49,31 @@ def factorial(n):
 
 
 
+def perm(n, k):
+    """Return P(n, k), the number of permutations of length k drawn from n
+    choices.
+    """
+    result = 1
+    assert k > 0
+    while k:
+        result *= n
+        n -= 1
+        k -= 1
+
+    return result
+
+
+
+
+def comb(n, k):
+    """Return C(n, k), the number of combinations (subsets)
+    of length k drawn from n choices.
+    """
+    return perm(n, k)//factorial(k)
+
+
+
+
 def norm_1(iterable):
     return sum(abs(x) for x in iterable)
 
@@ -274,6 +299,21 @@ def one(iterable):
 
 
 
+def is_single_valued(iterable, equality_pred=operator.eq):
+    it = iter(iterable)
+    try:
+        first_item = it.next()
+    except StopIteration:
+        raise ValueError, "empty iterable passed to 'single_valued()'"
+
+    for other_item in it:
+        if not equality_pred(other_item, first_item):
+            return False
+    return True
+
+
+
+
 def single_valued(iterable, equality_pred=operator.eq):
     """Return the first entry of *iterable*; Assert that other entries
     are the same with the first entry of *iterable*.
@@ -320,21 +360,57 @@ FunctionValueCache = memoize
 
 @my_decorator
 def memoize_method(method, instance, *args):
-    dicname = "_memoize_dic_"+method.__name__
+    cache_dict_name = intern("_memoize_dic_"+method.__name__)
     try:
-        return getattr(instance, dicname)[args]
+        return getattr(instance, cache_dict_name)[args]
     except AttributeError:
         result = method(instance, *args)
-        setattr(instance, dicname, {args: result})
+        setattr(instance, cache_dict_name, {args: result})
         return result
     except KeyError:
         result = method(instance, *args)
-        getattr(instance,dicname)[args] = result
+        getattr(instance, cache_dict_name)[args] = result
         return result
 
 
 
 
+def memoize_method_nested(inner):
+    """Adds a cache to a function nested inside a method. The cache is attached
+    to *memoize_cache_context* (if it exists) or *self* in the outer (method)
+    namespace.
+
+    Requires Python 2.5 or newer.
+    """
+
+    from functools import wraps
+    cache_dict_name = intern("_memoize_inner_dic_%s_%s_%d"
+            % (inner.__name__, inner.func_code.co_filename,
+                inner.func_code.co_firstlineno))
+
+    from inspect import currentframe, getouterframes
+    outer_frame = getouterframes(currentframe())[1][0]
+    cache_context = outer_frame.f_locals.get("memoize_cache_context")
+    if cache_context is None:
+        cache_context = outer_frame.f_locals.get("self")
+
+    try:
+        cache_dict = getattr(cache_context, cache_dict_name)
+    except AttributeError:
+        cache_dict = {}
+        setattr(cache_context, cache_dict_name, cache_dict)
+
+    @wraps(inner)
+    def new_inner(*args):
+        try:
+            return cache_dict[args]
+        except KeyError:
+            result = inner(*args)
+            cache_dict[args] = result
+            return result
+
+    return new_inner
+
 FunctionValueCache = memoize
 
 # }}}
@@ -683,6 +759,58 @@ def uniform_interval_splitting(n, granularity, max_intervals):
 
 
 
+def find_max_where(predicate, prec=1e-5, initial_guess=1, fail_bound=1e38):
+    """Find the largest value for which a predicate is true,
+    along a half-line. 0 is assumed to be the lower bound."""
+
+    # {{{ establish bracket
+
+    mag = 1
+
+    if predicate(mag):
+        mag *= 2
+        while predicate(mag):
+            mag *= 2
+
+            if mag > fail_bound:
+                raise RuntimeError("predicate appears to be true "
+                        "everywhere, up to %g" % fail_bound)
+
+        lower_true = mag/2
+        upper_false = mag
+    else:
+        mag /= 2
+
+        while not predicate(mag):
+            mag /= 2
+
+            if mag < prec:
+                return mag
+
+        lower_true = mag
+        upper_false = mag*2
+
+    # }}}
+
+    # {{{ refine
+
+    # Refine a bracket between *lower_true*, where the predicate is true,
+    # and *upper_false*, where it is false, until *prec* is satisfied.
+
+    assert predicate(lower_true)
+    assert not predicate(upper_false)
+
+    while abs(lower_true-upper_false) > prec:
+        mid = (lower_true+upper_false)/2
+        if predicate(mid):
+            lower_true = mid
+        else:
+            upper_false = mid
+    else:
+        return lower_true
+
+    # }}}
+
 # }}}
 
 # {{{ argmin, argmax ----------------------------------------------------------
@@ -1138,6 +1266,7 @@ def a_star(initial_state, goal_state, neighbor_map,
 # }}}
 
 # {{{ formatting --------------------------------------------------------------
+
 # {{{ table formatting --------------------------------------------------------
 class Table:
     """An ASCII table generator."""
@@ -1243,6 +1372,7 @@ def word_wrap(text, width, wrap_using="\n"):
 # }}}
 
 # {{{ command line interfaces -------------------------------------------------
+
 class CPyUserInterface(object):
     class Parameters(Record):
         pass
@@ -1318,6 +1448,7 @@ class CPyUserInterface(object):
 # }}}
 
 # {{{ code maintenance --------------------------------------------------------
+
 class MovedFunctionDeprecationWrapper:
     def __init__(self, f):
         self.f = f
@@ -1333,6 +1464,7 @@ class MovedFunctionDeprecationWrapper:
 # }}}
 
 # {{{ debugging ---------------------------------------------------------------
+
 def typedump(val, max_seq=5, special_handlers={}):
     try:
         hdlr = special_handlers[type(val)]
@@ -1346,8 +1478,13 @@ def typedump(val, max_seq=5, special_handlers={}):
     except TypeError:
         return type(val).__name__
     else:
+        if isinstance(val, dict):
+            return "{%s}" % (
+                    ", ".join(
+                        "%r: %s" % (str(k), typedump(v))
+                        for k, v in val.iteritems()))
+
         try:
-            l = len(val)
             if len(val) > max_seq:
                 return "%s(%s,...)" % (
                         type(val).__name__,
@@ -1464,16 +1601,6 @@ def assert_not_a_file(name):
         raise IOError, "file `%s' already exists" % name
 
 
-def _test():
-    import doctest
-    doctest.testmod()
-
-if __name__ == "__main__":
-    _test()
-
-
-
-
 def add_python_path_relative_to_script(rel_path):
     import sys
     from os.path import dirname, join, abspath
@@ -1537,4 +1664,14 @@ def match_precision(dtype, dtype_to_match):
 
 # }}}
 
+
+
+
+def _test():
+    import doctest
+    doctest.testmod()
+
+if __name__ == "__main__":
+    _test()
+
 # vim: foldmethod=marker
